import createDebug from 'debug';
import Listr from 'listr';

import fs from 'mz/fs';

import url from 'url';
import path from 'path';

import { CheckExistDir, CheckCreation } from './fs';

import { getHtml, loadFiles } from './loader';

import { basicNames, exchangeToNativLinks, fullLink } from './name';
import { localLinks } from './parse';

const debug = createDebug('page-loader');

const loadPage = (page, dir) => {
  debug('start load a page %s', page);

  const names = basicNames(page, dir);

  return CheckExistDir(dir)
    .then(() => CheckCreation(names.filepath))
    .then(() => getHtml(page))
    .then((html) => {
      const exchanges = exchangeToNativLinks(localLinks(html), names.simpleDirFiles);

      const nativeHtml = exchanges.reduce((str, exch) =>
        str.replace(new RegExp(exch.link, 'g'), exch.nativeLink), html);
      debug('local links location changed');

      if (exchanges.length === 0) {
        return nativeHtml;
      }
      const { protocol, hostname } = url.parse(page);
      const urlData = { protocol, hostname };
      const exchangesToLoad = exchanges.map((exch) => {
        const exchToLoad = {
          link: fullLink(exch.link, urlData),
          path: path.join(dir, exch.nativeLink),
        };
        return exchToLoad;
      });

      return CheckCreation(names.dirFiles)
        .then(() => fs.mkdir(names.dirFiles))
        .then(() => debug('created a load directory %s', dir))
        .then(() => loadFiles(exchangesToLoad))
        .then(() => nativeHtml);
    })
    .then(html => fs.writeFile(names.filepath, html))
    .then(() => debug('loaded a page %s', page))
    .then(() => debug('end load a page %s', page))
    .catch((err) => {
      debug(`page ${page} not found due to reason: ${err.message}`);
      throw err;
    });
};

export default (page, dir) => {
  const tasks = new Listr([{
    title: `load a page ${page}`,
    task: () => loadPage(page, dir),
  }], { concurrent: true });

  return tasks.run();
};
